%%
% LPV Example
%
% M-file to analyze an uncetrain linear parameter varying system using IQCs 
% and dissipation inequalities. The analysis is performed for various bounds
% on the parameter variation rate and compared to the wcgain command
% The LPV system is taken from "Time-Delay System", Weehon Tan, Berkeley, 
% Automatica vol33, 1997, pp411-419
% In order to run the example Yalmip and SDPT3 are required


%% 
% Setting up the plant model
clear all
yvec = (2:1:7); %scheduling parameter vector
K = sqrt(4.8*yvec-8.6); %parameter varying gain of the 1st order LPV system
T = sqrt(133.6-16.8*yvec); %time constant

% Adding a basic gain scheduled PI controller
zeta = .7; %CL damping requirement at each grid point
omega = .25; %CL frequency requirement at each grid point

Ki = omega.^2.*T./K; %Integrator gain
Kp = (2*zeta*omega.*T-1)./K; %proportinal gain

%build LPV plant, controller and closed loop
for ii = 1:length(yvec)
    P(:,:,ii)= ss(-1/T(ii),1/(T(ii)),K(ii),0);
    
    C(:,:,ii) = ss(0,Ki(ii),1,Kp(ii));
    
    clp(:,:,ii) = feedback(P(:,:,ii)*C(:,:,ii),1);
    
end

%time delay at the plant intput
Td = .5;
tdelay = tf([Td^2/12,-Td/2,1],[Td^2/12,Td/2,1]);


%% unbounded rates
clear sens M lyap gam

%options for sdp solver
opts.sdp = sdpsettings('solver','sdpt3','verbose',0,...
    'sdpt3.maxit',150);

%specifiying IQC(Psi,M) for norm bounded uncertainty at the plant input
psi = eye(2);

% uncertainty is norm bounded by kappa
kappavec = [0,0.01,0.05,0.10,.15,.2,.25,.3,.35,.4,.45,.5];
for jj = 1:length(kappavec)
    kappa=kappavec(jj);
    M{1} = blkdiag(kappa^2,-1);
    
    %build extended plant (including IQC filter psi)
    plant = P*tdelay;
    controller = C;
    systemnames = 'plant controller psi';
    inputvar = '[ w2{1}; d{1} ]';
    outputvar = '[ psi; d-plant(1) ]';
    input_to_plant = '[w2+controller]';
    input_to_controller = '[d-plant(1) ]';
    input_to_psi = '[controller; w2]';
    sysoutname = 'sens';
    cleanupsysic = 'yes';
    sysic
    sens.OutputName = {'e','z1', 'z2'};
    
    lpvsys.ss = ssbal(sens(:,:,:));
    lpvsys.pvalues = yvec';
    lpvsys.prates = []; %no bounds on the rates
    
    %defining basis functions for Lyapunov matrix
    lyap.basfct{1} = @(p)1; %constant
    lyap.deriv{1,1} = @(p)0;
    
    %run performance analysis
    [gam(jj),info] = robustlpvperf(lpvsys,1,M,lyap,opts);

end

%plot results
last = 4;
figure(1)
plot(kappavec(1:last),gam(1:last),'+--','color',mycolor('maroon'),'MarkerSize',12)
xlim([0 0.5])
xlabel('norm bound on uncertainty b [-]')
ylabel('worst case gain \gamma [-]')
if exist('garyfyFigure','file')
    garyfyFigure;
end
box on

% matlab2tikz('wcgainsunbounded.tex')


%% influence of basis functions on the performance results
clear sens M lyap gam

%options for sdp solver
opts.sdp = sdpsettings('solver','sdpt3','verbose',0,...
    'sdpt3.maxit',150);

%specifiying IQC(Psi,M) for norm bounded uncertainty at the plant input
psi = eye(2);

% uncertainty is norm bounded by kappa
kappavec = [0,0.01,0.05,0.10,.15,.2,.25,.3,.35,.4,.45,.5];
for jj = 1:length(kappavec)
    kappa=kappavec(jj);
    M{1} = blkdiag(kappa^2,-1);
    
    %build extended plant (including IQC filter psi)
    plant = P*tdelay;
    controller = C;
    systemnames = 'plant controller psi';
    inputvar = '[ w2{1}; d{1} ]';
    outputvar = '[ psi; d-plant(1) ]';
    input_to_plant = '[w2+controller]';
    input_to_controller = '[d-plant(1) ]';
    input_to_psi = '[controller; w2]';
    sysoutname = 'sens';
    cleanupsysic = 'yes';
    sysic
    sens.OutputName = {'e','z1', 'z2'};
    
    for ii= 1:2
        
        lpvsys.ss = ssbal(sens(:,:,:));
        lpvsys.pvalues = yvec';
        lpvsys.prates = [-2;2]; %rate bound
       
        clear lyap

        %defining basis functions for Lyapunov matrix
        lyap.basfct{1} = @(p)1; %constant term
        lyap.deriv{1,1} = @(p)0;
        
        lyap.basfct{2} = @(p)p; %linear term
        lyap.deriv{2,1} = @(p)1;
        
        if ii == 2
            lyap.basfct{3} = @(p)p^2; %quadratic term
            lyap.deriv{3,1} = @(p)2*p;
        end
        
        %run performance analysis
        [gam(ii,jj),info] = robustlpvperf(lpvsys,1,M,lyap,opts);
    end
    
end

%plot results
figure(2)
hold on
plot(kappavec(1:7),gam(1,1:7),'+--','color',mycolor('maroon'),'MarkerSize',12)
plot(kappavec(1:8),gam(2,1:8),'x-','color',mycolor('coolblue'),'MarkerSize',12)
xlim([0 0.5])
xlabel('norm bound on uncertainty b [-]')
ylabel('worst case gain \gamma [-]')
legend('P affine','P quadratic');
if exist('garyfyFigure','file')
    garyfyFigure;
end
legend('Location','SouthEast')
box on

% matlab2tikz('wcgainsbounded.tex')


%% comparison with worst case
clear sens

% uncertainty is norm bounded by kappa
kappavec = [0.01,0.05,0.10,.15,.2,.25,.3,.35,.4,.45,.5];

%at each grid point compute worst case LTI gain using wcgain.m
for jj = 1:length(kappavec)
    kappa=kappavec(jj);
    clear delta
    delta = ultidyn('delta',[1,1],'Bound',kappa);
    for ii = 1:length(yvec)
        
        plant =P(:,:,ii)*tdelay;;
        controller = C(:,:,ii);
        
        systemnames = 'plant controller delta';
        inputvar = '[ d{1} ]';
        outputvar = '[ d-plant(1) ]';
        input_to_plant =  '[controller+delta]';
        input_to_controller = '[d-plant(1) ]';
        input_to_delta = '[controller]';
        sysoutname = 'sensii';
        cleanupsysic = 'yes';
        sysic
        
        [wcg{ii},wcu{ii},info] = wcgain(sensii);
        
        wcgjj(ii) = wcg{ii}.UpperBound;
        
    end
    
    %worst case gain for a given kappa over all grid points
    wcglti(jj+1) = max(wcgjj);
end
clear sens M lyap gam

%options for sdp solver
opts.sdp = sdpsettings('solver','sdpt3','verbose',0,...
    'sdpt3.maxit',150);

%specifiying IQC(Psi,M) for norm bounded uncertainty at the plant input
psi = eye(2);

% uncertainty is norm bounded by kappa
kappavec = [0,0.01,0.05,0.10,.15,.2,.25,.3,.35,.4,.45,.5];
for jj = 1:length(kappavec)
    kappa=kappavec(jj);
    M{1} = blkdiag(kappa^2,-1);
    
    %build extended plant (including IQC filter psi)
    plant = P*tdelay;
    controller = C;
    systemnames = 'plant controller psi';
    inputvar = '[ w2{1}; d{1} ]';
    outputvar = '[ psi; d-plant(1) ]';
    input_to_plant = '[w2+controller]';
    input_to_controller = '[d-plant(1) ]';
    input_to_psi = '[controller; w2]';
    sysoutname = 'sens';
    cleanupsysic = 'yes';
    sysic
    sens.OutputName = {'e','z1', 'z2'};
    
    for ii= 1:2
        
        lpvsys.ss = ssbal(sens(:,:,:));
        lpvsys.pvalues = yvec';
        lpvsys.prates = [-.1;.1]; %rate bound
        
        clear lyap

        %defining basis functions for Lyapunov matrix
        lyap.basfct{1} = @(p)1; %constant term
        lyap.deriv{1,1} = @(p)0;
        
        lyap.basfct{2} = @(p)p; %linear term
        lyap.deriv{2,1} = @(p)1;
        
        if ii == 2
            lyap.basfct{3} = @(p)p^2; %quadratic term
            lyap.deriv{3,1} = @(p)2*p;
        end
        
        %run performance analysis               
        [gam(ii,jj),info] = robustlpvperf(lpvsys,1,M,lyap,opts);
    end
    
end

%plot results
figure(3)
hold on
plot(kappavec,gam(1,:),'+--','color',mycolor('maroon'),'MarkerSize',12)
plot(kappavec,gam(2,:),'x-','color',mycolor('coolblue'),'MarkerSize',12)
plot(kappavec(2:end),wcglti(2:end),'o:','color',mycolor('darkgray'),'MarkerSize',8)
xlim([0 0.5])
xlabel('norm bound on uncertainty b [-]')
ylabel('worst case gain \gamma [-]')
legend('P affine','P quadratic','wcgain');
if exist('garyfyFigure','file')
    garyfyFigure;
end
box on
legend('Location','NorthWest')
% matlab2tikz('wcgainsLTI.tex')

