% function  [gam,info] = robustlpvperf(lpvsys,nw,M,lyap,opts)
%
% Computes the worst case induced L2-norm gain of an LPV system under IQC 
% uncertainties. 
% lpvsys contains the filter Psi describing the uncertainty

function [gam,info] = robustlpvperf(lpvsys,nw,M,lyap,opts)

% values of the scheduling parameter and rates in vector form
% prates describes the vertices, e.g. for 2 parameters with bound 1 
% [-1 -1; -1 1; 1 -1; 1 1]
pvalues = lpvsys.pvalues;
prates = lpvsys.prates;
[ngrid,npar] = size(pvalues);

%if no rates specified assume 0
if isempty(prates) 
    prates = zeros(1,npar);
end

%nz is Vector containing the outputs of Psi, e.g. nz(1) output of Psi1
nz = cellfun(@length,M);
[nout,nin] = size(lpvsys.ss);
%nd: performance inputs
%ne: performance outputs
nd = nin-nw; ne = nout - sum(nz);
ns = order(lpvsys.ss(:,:,1));
[a,b,c,d] = ssdata(ssbal(lpvsys.ss));

% seperate system matrices
% [dotx]   [a  b1  b2  ] [x]
% [ z  ] = [c1 d11 d12 ]*[w]
% [ e  ]   [c2 d21 d22 ] [d]
b1 = b(:,1:nw,:);
b2 = b(:,nw+1:end,:);

c2 = c(sum(nz)+1:end,:,:);
d21 = d(sum(nz)+1:end,1:nw,:);
d22 = d(sum(nz)+1:end,nw+1:end,:);

% initialize coefficient matrices for X
lmis = [];
for ii = 1:length(lyap.basfct)
    X{ii} = sdpvar(ns,ns,'symmetric');
end
% initialize perfomance index gamma
gam = sdpvar(1,1);

cdmat = [c,d];
for ii = 1:length(M)
    % initialize lambda > 0 for each M
    lam{ii} = sdpvar(1,1);
    lmis=set([lmis,lam{ii} >=0]);
    % extract rows for z from [C D]
    if ii == 1
        zmat(:,:,:,ii) = cdmat(1:nz(ii),:,:);
    else
        zmat(:,:,:,ii) = cdmat(sum(nz(1:ii-1))+1:sum(nz(1:ii-1))+nz(ii),:,:);
    end
end


%set lmis over all grid points
for ii = 1:ngrid
    aii = a(:,:,ii);
    b1ii = b1(:,:,ii);
    b2ii = b2(:,:,ii);
    
    c2ii = c2(:,:,ii);
    
    d21ii = d21(:,:,ii);
    d22ii = d22(:,:,ii);
    
    Xii = evalX(X,lyap.basfct,pvalues(ii,:));
    lmis=set([lmis,Xii >=0]); %not good if X const.
    
    %loop over rates
    for jj = 1:length(prates(:,1))
        
        dXdtii = evaldXdt(X,lyap.deriv,pvalues(ii,:),prates(jj,:));
    
        lmibrliqc{ii,jj} = [Xii*aii+aii'*Xii + dXdtii, Xii*b1ii, Xii*b2ii, c2ii'; ...
            b1ii'*Xii, zeros(nw,nw), zeros(nw,nd) , d21ii'; ...
            b2ii'*Xii, zeros(nd,nw), -gam*eye(nd) , d22ii';...
            c2ii, d21ii, d22ii, -gam*eye(ne)];
        
        for iM = 1:length(M)
            lmibrliqc{ii,jj} = lmibrliqc{ii,jj} + ...
            lam{iM}*[zmat(:,:,ii,iM)'; zeros(nz(iM),ne)']*M{iM}* ...
            [zmat(:,:,ii,iM),zeros(nz(iM),ne)];

        end
        
        lmis = set([lmis,lmibrliqc{ii,jj}<=0]);
    end
        
%     lmibrliqc{ii} = [X*aii+aii'*X, X*b2ii, X*b1ii, c1ii', c2ii'; ...
%         b2ii'*X, zeros(nw,nw), zeros(nw,nd) , d12ii', d22ii'; ...
%         b1ii'*X, zeros(nd,nw), -gam*eye(nd) , d11ii', d21ii';...
%         c1ii, d12ii, d11ii, -gam*eye(ne), zeros(nz,ne)';...
%         c2ii, d22ii, d21ii,zeros(nz,ne), -lam*inv(M)];
    
    
    
end


info = solvesdp(lmis,gam,opts.sdp);
for ii = 1:length(lyap.basfct)
    info.X{ii} = double(X{ii});
end
for ii = 1:length(M)
    info.lam{ii} = double(lam{ii});

end
gam = double(gam);



end


%evaluate lyapunov matrix at point pvalue
function Xval = evalX(X,basfct,pvalue)

Xval = zeros(size(X{1}));
for ii = 1:length(X)
    Xval = Xval+ X{ii}*basfct{ii}(pvalue);
end

end

%evaluate lyapunov matrix at point pvalue and prates
function dXdtval = evaldXdt(X,deriv,pvalue,prates)

dXdtval = zeros(size(X{1}));
for ii = 1:length(X)
    for jj = 1:length(pvalue)
    dXdtval = dXdtval+ X{ii}*deriv{ii,jj}(pvalue)*prates(jj);
    end
end

end